

#include "can.h"
#include "can_driver.h"
#include "driverlib.h"
#include "device.h"

uint16_t txMsgData[8];
uint16_t rxMsgData[8];

__interrupt void canISR(void);

volatile uint32_t errorFlag = 0;

void Can_Init(void)
{

    Device_init();    // disables watchdog also.
    Device_initGPIO();

    GPIO_setPinConfig(DEVICE_GPIO_CFG_CANRXA);
    GPIO_setPinConfig(DEVICE_GPIO_CFG_CANTXA);

   CAN_selectClockSource(CANA_BASE, CAN_CLOCK_SOURCE_SYS);

    CAN_initModule(CANA_BASE);
    CAN_setBitRate(CANA_BASE, DEVICE_SYSCLK_FREQ, 500000, 16);

    CAN_enableInterrupt(CANA_BASE, CAN_INT_IE0 | CAN_INT_ERROR | CAN_INT_STATUS);
    Interrupt_initModule();
    Interrupt_initVectorTable();
    EINT;
    ERTM;

    Interrupt_register(INT_CANA0, &canISR);
    Interrupt_enable(INT_CANA0);
    CAN_enableGlobalInterrupt(CANA_BASE, CAN_GLOBAL_INT_CANINT0);

    //Configure CAN ID : 0x701 as Transmit ID associated with Msgobject 1
    CAN_setupMessageObject(CANA_BASE, CAN_MSG_OBJECT_1, CAN_TX_MSG_ID_1, CAN_MSG_FRAME_STD, CAN_MSG_OBJ_TYPE_TX, CAN_MSG_ID_MASK, CAN_MSG_OBJ_TX_INT_ENABLE, MSG_DATA_LENGTH_EIGHT);

    // Configure CAN ID : 0x717 as receive  ID associated with Msgobject 2
    CAN_setupMessageObject(CANA_BASE, CAN_MSG_OBJECT_2, CAN_RX_MSG_ID_1, CAN_MSG_FRAME_STD, CAN_MSG_OBJ_TYPE_RX, CAN_MSG_ID_MASK, CAN_MSG_OBJ_RX_INT_ENABLE, MSG_DATA_LENGTH_EIGHT);

    CAN_startModule(CANA_BASE);

}

void CAN_MainTask(void)
{
    //uint16_t txMsgData[8], rxMsgData[8];

    txMsgData[0] = 0x01;
    txMsgData[1] = 0x02;
    txMsgData[2] = 0x03;
    txMsgData[3] = 0x04;
    txMsgData[4] = 0x05;
    txMsgData[5] = 0x06;
    txMsgData[6] = 0x07;
    txMsgData[7] = 0x08;

    for(;;)
    {
       CAN_sendMessage(CANA_BASE, CAN_MSG_OBJECT_1, MSG_DATA_LENGTH_EIGHT, txMsgData);

       DEVICE_DELAY_US(5000000);   // 5 seconds

       txMsgData[0] += 0x01;
       txMsgData[1] += 0x01;
       txMsgData[2] += 0x01;
       txMsgData[3] += 0x01;
       txMsgData[4] += 0x01;
       txMsgData[5] += 0x01;
       txMsgData[6] += 0x01;
       txMsgData[7] += 0x01;

       if(txMsgData[7] > 0xFF)
       {
          txMsgData[0] = 0;
          txMsgData[1] = 0;
          txMsgData[2] = 0;
          txMsgData[3] = 0;
          txMsgData[4] = 0;
          txMsgData[5] = 0;
          txMsgData[6] = 0;
          txMsgData[7] = 0;
        }
    }
}

void Can_Transmit(void)
{
}

void Can_Rx_Interrupt(void)
{
}

__interrupt void
canISR(void)
{
   uint32_t status;
   status = CAN_getInterruptCause(CANA_BASE);

   if(status == CAN_INT_INT0ID_STATUS)
   {
      status = CAN_getStatus(CANA_BASE);

      if(((status  & ~(CAN_STATUS_TXOK | CAN_STATUS_RXOK)) != 7) &&
         ((status  & ~(CAN_STATUS_TXOK | CAN_STATUS_RXOK)) != 0))
      {
         errorFlag = 1;
      }
   }
   else if(status == CAN_MSG_OBJECT_1)   //1
   {
      CAN_clearInterruptStatus(CANA_BASE, CAN_MSG_OBJECT_1);
      errorFlag = 0;
   }
   else if(status == CAN_MSG_OBJECT_2)   //2
   {
      CAN_readMessage(CANA_BASE, CAN_MSG_OBJECT_2, rxMsgData);
      CAN_clearInterruptStatus(CANA_BASE, CAN_MSG_OBJECT_2);
      errorFlag = 0;
   }
   else
   {
   }

   CAN_clearGlobalInterruptStatus(CANA_BASE, CAN_GLOBAL_INT_CANINT0);
   Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP9);

}
